/* Smart control Utility */
/*

Copyright  1999 The Regents of the University of California. 
All Rights Reserved. 

Permission to use, copy, modify, and distribute this software 
and its documentation for educational, research and non-profit 
purposes, without fee, and without a written agreement is hereby 
granted, provided that the above copyright notice, this 
paragraph and the following three paragraphs appear in all copies. 

Permission to incorporate this software into commercial products 
may be obtained by contacting the University of California 
at: Jan Lester,368 Applied Sciences Building, UCSC, Santa Cruz, 
CA 95064 (831) 459-4533. 

This software program and documentation are copyrighted by The 
Regents of the University of California. The software program 
and documentation are supplied "as is", without any accompanying 
services from The Regents. The Regents does not warrant that the
operation of the program will be uninterrupted or error-free. The 
end-user understands that the program was developed for research 
\purposes and is advised not to rely exclusively on the program for 
any reason. 

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY 
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE 
AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN 
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA 
SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS. 


*/

#include <stdio.h>
#include <sys/ioctl.h>
#include <linux/hdreg.h>
#include <sys/fcntl.h>
#include <sys/types.h>
#include <string.h>

#include "smart.h"
#include "smartprint.h"
#include "smartctl.h"


/*  void Usage (void) 
	prints help information for command syntax */



void Usage ( void)
{

	printf( "smartctl version %i.%i - S.M.A.R.T. Control Program\n", 
			VERSION_MAJOR, VERSION_MINOR);
	printf( "useage: smartctl -[opts] [device]\n");
	printf( "Read Only Commands:\n");
	printf( "\t\t%c\t\tShow All S.M.A.R.T. Information\n", SMARTVERBOSEALL);
	printf( "\t\t%c\t\tShow General S.M.A.R.T. Attributes\n", GENERALSMARTVALUES);
	printf( "\t\t%c\t\tShow Vendor S.M.A.R.T. Attributes\n", SMARTVENDORATTRIB);
	printf( "\t\t%c\t\tShow S.M.A.R.T. Drive Error Log\n", SMARTERRORLOG);
	printf( "\t\t%c\t\tShow S.M.A.R.T. Drive SelfTest Log\n", SMARTSELFTESTLOG);
	printf( "\t\t%c\t\tShow S.M.A.R.T. Drive Info\n", DRIVEINFO);
	printf( "\t\t%c\t\tCheck S.M.A.R.T. Status\n", CHECKSMART);
	printf( "\n");
	printf( "Enable / Disable Commands:\n");
	printf( "\t\t%c\t\tEnable S.M.A.R.T. data collection\n", SMARTENABLE);
	printf( "\t\t%c\t\tDisable S.M.A.R.T.data collection\n", SMARTDISABLE);
	printf( "\t\t%c\t\tEnable S.M.A.R.T. Automatic Offline Test\n", SMARTAUTOOFFLINEENABLE);
	printf( "\t\t%c\t\tDisable S.M.A.R.T. Automatic Offline Test\n", SMARTAUTOOFFLINEDISABLE);
	printf( "\n");
	printf( "Test Commands:\n");
	printf( "\t\t%c\t\tExecute Off-line data collection\n", SMARTEXEOFFIMMEDIATE);
	printf( "\t\t%c\t\tExecute Short Self Test\n", SMARTSHORTSELFTEST );
	printf( "\t\t%c\t\tExecute Short Self Test (Captive Mode)\n", SMARTSHORTCAPSELFTEST );
	printf( "\t\t%c\t\tExecute Extended Self Test\n", SMARTEXTENDSELFTEST );
	printf( "\t\t%c\t\tExecute Extended Self Test (Captive Mode)\n", SMARTEXTENDCAPSELFTEST );
	printf( "\t\t%c\t\tExecute Self Test Abort \n", SMARTSELFTESTABORT );

}
	

/*  void ParseOpts ( chars *opts) 
	Takes command options and sets features to be run */	

void ParseOpts ( char *opts)
{
	int i = 0;
	char options[255];
	
	strcpy ( (char *) &options, opts);

	if ( options[0] != '-' )
	{
		Usage();
		exit (-1);
	}

	for (i = 1; options[i] != '\0'; i++ )
	{
	
		switch (options[i])
		{

		case DRIVEINFO :
			driveinfo  = TRUE;
			break;		

		case CHECKSMART :
			driveinfo  = TRUE;
			checksmart = TRUE;		
			break;

		case SMARTVERBOSEALL :
			driveinfo			   = TRUE;
			checksmart			   = TRUE;
			generalsmartvalues     = TRUE;
			smartvendorattrib      = TRUE;
			smarterrorlog          = TRUE;
			smartselftestlog       = TRUE;
			break;

		case SMARTVENDORATTRIB :
			smartvendorattrib      = TRUE;
			break;

		case GENERALSMARTVALUES :
			generalsmartvalues     = TRUE;
			break;

		case SMARTERRORLOG :
			smarterrorlog          = TRUE;
			break;

		case SMARTSELFTESTLOG :
			smartselftestlog       = TRUE;
			break;

		case SMARTDISABLE :
			smartdisable           = TRUE;
			break;

		case SMARTENABLE :
			smartenable			   = TRUE;
			break;

		case SMARTAUTOOFFLINEENABLE: 
			smartautoofflineenable = TRUE;
			break;

		case SMARTAUTOOFFLINEDISABLE:
			smartautoofflinedisable = TRUE;
			break;

		case SMARTEXEOFFIMMEDIATE:
			smartexeoffimmediate = TRUE;
			break;

		case SMARTSHORTSELFTEST :
			smartshortselftest   = TRUE;
			break;
	
		case SMARTEXTENDSELFTEST :
			smartextendselftest  = TRUE;
			break;

		case  SMARTSHORTCAPSELFTEST:
			smartshortcapselftest = TRUE;
			break;

		case SMARTEXTENDCAPSELFTEST:
			smartextendcapselftest   = TRUE;
			break;

		case SMARTSELFTESTABORT:
			smartselftestabort   = TRUE;
			break;
		
                default:
			Usage();
			exit (-1);	
		}

		if ( (smartexeoffimmediate + smartshortselftest +
			smartextendselftest + smartshortcapselftest +
			smartextendcapselftest ) > 1)
		{
			Usage();
			printf ("\n ERROR: smartctl can only run a single test at a time \n");
			exit(-1);
		}

	
	}


}


/* Main Program */

int main (int argv, char **argc)
{
	
	int fd;
	char *options, *device;
	struct hd_driveid drive;
	
	struct ata_smart_values smartval;
	struct ata_smart_thresholds smartthres;
	struct ata_smart_errorlog smarterror;
	struct ata_smart_selftestlog smartselftest;
	
	if ( argv != 3 )
	{
		Usage();
		exit (-1);
	}
	
	
	options = *(argc+1);
	device = *(argc+2);

	
	ParseOpts ( options);

	
	/* open device */
	
	fd = open ( device, O_RDWR );

	if ( fd < 0) {
		perror ( "Device open failed");
		exit(-1);
	}
	

	if ( driveinfo )
	{

		if (  ataReadHDIdentity ( fd, &drive) != 0 )
		{
			printf("Smartctl: Hard Drive Identity Failed\n");
			exit(0);	
		}
		
		ataPrintDriveInfo(drive); 
		


		switch (ataSmartSupport(drive))
		{
			case 0:
				printf ("Drive does not support S.M.A.R.T.\n");
				exit (0);
				break;
			case 1:
				printf ("Drive supports S.M.A.R.T. and is disabled\n");
				printf( "Use option -%c to enable\n", SMARTENABLE );
				exit (0);
				break;
			case 2: 
				printf ("Drive supports S.M.A.R.T. and is enabled\n");
				break;
			case 255:
				printf ("Drive supports S.M.A.R.T. and is ");
				if ( ataSmartStatus(fd) != 0) 
				{
					printf( "disabled\n");
					printf( "Use option -%c to enable\n", SMARTENABLE );
					exit(0);
				}
				else
				{
					printf( "enabled\n");
				}
				break;
			default :
				/* not used */
				break;
		}
	}





	if ( smartdisable )
	{
		
		if ( ataDisableSmart(fd) != 0) 
		{
			printf( "Smartctl: Smart Enable Failed\n");
			exit(-1);
		}
		printf("S.M.A.R.T. Disabled\n");
		exit (0);
		
	}


	if ( smartenable )
	{
		if ( ataEnableSmart(fd) != 0) 
		{
			printf( "Smartctl: Smart Enable Failed\n");
			exit(-1);
		}
	
		/* printstatus here */
	}

	
	
	/* for everything else read values and thresholds 
	   are needed */	

	if ( ataReadSmartValues ( fd, &smartval) != 0 )
	{
		printf("Smartctl: Smart Values Read Failed\n");
		exit (-1);
	}

	if ( ataReadSmartThresholds ( fd, &smartthres) != 0 )
	{
		printf("Smartctl: Smart THresholds Read Failed\n");
	}
	
	
	if ( checksmart )
	{
		/* pseudo is used because linux does not support access to
		   Task Fiule registers */
		
		ataPsuedoCheckSmart ( smartval , smartthres);

	}

	
	if (  generalsmartvalues )
	{
		ataPrintGeneralSmartValues( smartval ); 
	}

	if ( smartvendorattrib )
	{
		PrintSmartAttribWithThres( smartval, smartthres);
	}

	
	if ( smarterrorlog )
	{
		if ( isSmartErrorLogCapable(smartval) == 0)
		{
			printf("Device does not support Error Logging\n");
		} 
		else
		{
			if ( ataReadErrorLog ( fd, &smarterror) != 0 )
			{
				printf("Smartctl: Smart Errorlog Read Failed\n");
			}
			else
			{
				ataPrintSmartErrorlog ( smarterror); 
		
			}
		}
	}
	

	if ( smartselftestlog )
	{
		if ( isSmartErrorLogCapable(smartval) == 0)
		{
			printf("Device does not support Self Test Logging\n");
		} 
		else
		{
			if (  ataReadSelfTestLog( fd, &smartselftest) != 0 )
			{
				printf("Smartctl: Smart Self Test log Read Failed\n");
			}
			else
			{
				ataPrintSmartSelfTestlog (smartselftest); 
			}
		}
	}


	if (  smartautoofflineenable  )
	{
		if ( !isSupportAutomaticTimer (smartval))
		{
			printf("Device does not support S.M.A.R.T. Automatic Timers\n");
			exit(-1);
		}

		if ( ataEnableAutoOffline (fd) != 0) 
		{
			printf( "Smartctl: Smart Enable Automatic Offline Failed\n");
			exit(-1);
		}
		printf ("S.M.A.R.T. Automatic Offline Testing Enabled every four hours\n");

	}

	if (  smartautoofflinedisable  )
	{
		if ( !isSupportAutomaticTimer (smartval))
		{
			printf("Device does not support S.M.A.R.T. Automatic Timers\n");
			exit(-1);
		}
			
		if ( ataDisableAutoOffline (fd) != 0) 
		{
			printf( "Smartctl: Smart Disable Automatic Offline Failed\n");
			exit(-1);
		}
		printf ("S.M.A.R.T. Automatic Offline Testing Disabled\n");

	}


	if ( smartexeoffimmediate )
	{
		if ( ataSmartOfflineTest (fd) != 0) 
		{
			printf( "Smartctl: Smart Offline Failed\n");
			exit(-1);
		}
		printf ("Drive Command Successful offline test has begun\n");
		printf ("Please wait %d seconds for test to complete\n", isOfflineTestTime(smartval) );
		
	        printf ("Use smartctl -%c to abort test\n", SMARTSELFTESTABORT);	
		
		exit (0);
	}

	
	if ( smartshortcapselftest )
	{
		if ( ! isSupportSelfTest(smartval) )
		{
			printf (" ERROR: device does not support Self-Test function\n");
			exit(-1);
		}
		
		if ( ataSmartShortCapSelfTest (fd) != 0) 
		{
			printf( "Smartctl: Smart Short Self Test Failed\n");
			exit(-1);
		}
		printf ("Drive Command Successful offline test has begun\n");
		printf ("Please wait %d minutes for test to complete\n", isShortSelfTestTime (smartval) );
		
	        printf ("Use smartctl -%c to abort test\n", SMARTSELFTESTABORT);	
		
		/* Make sure Offline testing is last thing done */
		exit (0);
	}



	if ( smartshortselftest )
	{
		if ( ! isSupportSelfTest(smartval) )
		{
			printf (" ERROR: device does not support Self-Test function\n");
			exit(-1);
		}
		
		if ( ataSmartShortSelfTest (fd) != 0) 
		{
			printf( "Smartctl: Smart Short Self Test Failed\n");
			exit(-1);
		}
		printf ("Drive Command Successful offline test has begun\n");
		printf ("Please wait %d minutes for test to complete\n", isShortSelfTestTime (smartval) );
		
	        printf ("Use smartctl -%c to abort test\n", SMARTSELFTESTABORT);	
		
		/* Make sure Offline testing is last thing done */
		exit (0);
	}
	
	if ( smartextendselftest )
	{
		
		if ( ! isSupportSelfTest(smartval) )
		{
			printf (" ERROR: device does not support Self-Test function\n");
			exit(-1);
		}


		if ( ataSmartExtendSelfTest (fd) != 0) 
		{
			printf( "S.M.A.R.T. Extendend Self Test Failed\n");
			exit(-1);
		}
		
		printf ("Drive Command Successful self test has begun\n");
		printf ("Please wait %d minutes for test to complete\n", isExtendedSelfTestTime(smartval) );
		
	        printf ("Use smartctl -%c to abort test\n", SMARTSELFTESTABORT);	
	
		exit (0);
	}

	
	if ( smartextendcapselftest )
	{
		
		if ( ! isSupportSelfTest(smartval) )
		{
			printf (" ERROR: device does not support self test function\n");
			exit(-1);
		}


		if ( ataSmartExtendCapSelfTest (fd) != 0) 
		{
			printf( "S.M.A.R.T. Extendend Self Test Failed\n");
			exit(-1);
		}
		
		printf ("Drive Command Successful captive extended self test has begun\n");
		printf ("Please wait %d minutes for test to complete\n", isExtendedSelfTestTime(smartval) );
	        printf ("Use smartctl -%c to abort test\n", SMARTSELFTESTABORT);	
		
		
		exit (0);
	}

	if ( smartselftestabort )
	{
		
		if ( ! isSupportSelfTest(smartval) )
		{
			printf (" ERROR: device does not support Self-Test function\n");
			exit(-1);
		}


		if ( ataSmartSelfTestAbort (fd) != 0) 
		{
			printf( "S.M.A.R.T. Self Test Abort Failed\n");
			exit(-1);
		}
		
		printf ("Drive Command Successful self test aborted\n");
        }		
	
return 0;
}




